/*
 * Copyright (C) 2012-2014 Freescale Semiconductor, Inc.
 *
 * Author: Fabio Estevam <fabio.estevam@freescale.com>
 * Author: Jason Liu <r64343@freescale.com>
 *
 * See file CREDITS for list of people who contributed to this
 */

#include <asm/arch/clock.h>
#include <asm/arch/imx-regs.h>
#include <asm/arch/iomux.h>
#include <asm/arch/mx6-pins.h>
#include <asm/mach-imx/spi.h>
#include <linux/errno.h>
#include <asm/gpio.h>
#include <asm/mach-imx/mxc_i2c.h>
#include <asm/mach-imx/iomux-v3.h>
#include <asm/mach-imx/boot_mode.h>
#include <asm/mach-imx/video.h>
#include <mmc.h>
#include <fsl_esdhc.h>
#include <miiphy.h>
#include <netdev.h>
#include <asm/arch/mxc_hdmi.h>
#include <asm/arch/crm_regs.h>
#include <asm/io.h>
#include <asm/arch/sys_proto.h>
#include <i2c.h>
#include <input.h>
#include <power/pmic.h>
#include <power/pfuze100_pmic.h>
#include "../common/pfuze.h"
#include <usb.h>
#include <usb/ehci-ci.h>
#include <asm/arch/mx6-ddr.h>
#if defined(CONFIG_MX6DL) && defined(CONFIG_MXC_EPDC)
#include <lcd.h>
#include <mxc_epdc_fb.h>
#endif
#ifdef CONFIG_SATA
#include <asm/mach-imx/sata.h>
#endif
#ifdef CONFIG_FSL_FASTBOOT
#include <fsl_fastboot.h>
#ifdef CONFIG_ANDROID_RECOVERY
#include <recovery.h>
#endif
#endif /*CONFIG_FSL_FASTBOOT*/

DECLARE_GLOBAL_DATA_PTR;

#define UART_PAD_CTRL  (PAD_CTL_PUS_100K_UP |			\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm |			\
	PAD_CTL_SRE_FAST  | PAD_CTL_HYS)

#define USDHC_PAD_CTRL (PAD_CTL_PUS_47K_UP |			\
	PAD_CTL_SPEED_LOW | PAD_CTL_DSE_80ohm |			\
	PAD_CTL_SRE_FAST  | PAD_CTL_HYS)

#define ENET_PAD_CTRL  (PAD_CTL_PUS_100K_UP |			\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm | PAD_CTL_HYS)

#define SPI_PAD_CTRL (PAD_CTL_HYS | PAD_CTL_SPEED_MED | \
		      PAD_CTL_DSE_40ohm | PAD_CTL_SRE_FAST)

#define I2C_PAD_CTRL  (PAD_CTL_PUS_100K_UP |			\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm | PAD_CTL_HYS |	\
	PAD_CTL_ODE | PAD_CTL_SRE_FAST)

#define EPDC_PAD_CTRL    (PAD_CTL_PKE | PAD_CTL_SPEED_MED |	\
	PAD_CTL_DSE_40ohm | PAD_CTL_HYS)

/*
#define OTG_ID_PAD_CTRL (PAD_CTL_PKE | PAD_CTL_PUE |		\
	PAD_CTL_PUS_47K_UP  | PAD_CTL_SPEED_LOW |		\
	PAD_CTL_DSE_80ohm   | PAD_CTL_SRE_FAST  | PAD_CTL_HYS)


#define I2C_PMIC	1

#define I2C_PAD MUX_PAD_CTRL(I2C_PAD_CTRL)

#define DISP0_PWR_EN	IMX_GPIO_NR(1, 21)

#define KEY_VOL_UP	IMX_GPIO_NR(1, 4)
*/

int dram_init(void)
{
	gd->ram_size = imx_ddr_size();
	return 0;
}

iomux_v3_cfg_t const uart1_pads[] = {
	IOMUX_PADS(PAD_SD3_DAT6__UART1_RX_DATA | MUX_PAD_CTRL(UART_PAD_CTRL)),
	IOMUX_PADS(PAD_SD3_DAT7__UART1_TX_DATA | MUX_PAD_CTRL(UART_PAD_CTRL)),
};

#if 0
static iomux_v3_cfg_t const enet_pads[] = {
	IOMUX_PADS(PAD_ENET_MDIO__ENET_MDIO	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_MDC__ENET_MDC	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_TXC__RGMII_TXC	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_TD0__RGMII_TD0	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_TD1__RGMII_TD1	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_TD2__RGMII_TD2	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_TD3__RGMII_TD3	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_TX_CTL__RGMII_TX_CTL	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_REF_CLK__ENET_TX_CLK	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_RXC__RGMII_RXC	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_RD0__RGMII_RD0	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_RD1__RGMII_RD1	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_RD2__RGMII_RD2	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_RD3__RGMII_RD3	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_RGMII_RX_CTL__RGMII_RX_CTL	| MUX_PAD_CTRL(ENET_PAD_CTRL)),
	/* AR8031 PHY Reset */
	IOMUX_PADS(PAD_ENET_CRS_DV__GPIO1_IO25	| MUX_PAD_CTRL(NO_PAD_CTRL)),
};
#endif

static iomux_v3_cfg_t const fec_pads[] = {
	IOMUX_PADS(PAD_ENET_MDC__ENET_MDC | MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_MDIO__ENET_MDIO | MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_CRS_DV__ENET_RX_EN | MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_RXD0__ENET_RX_DATA0 | MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_RXD1__ENET_RX_DATA1 | MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_TX_EN__ENET_TX_EN | MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_TXD0__ENET_TX_DATA0 | MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_TXD1__ENET_TX_DATA1 | MUX_PAD_CTRL(ENET_PAD_CTRL)),
	IOMUX_PADS(PAD_GPIO_16__ENET_REF_CLK | MUX_PAD_CTRL(ENET_PAD_CTRL) | ((iomux_v3_cfg_t)IOMUX_CONFIG_SION << MUX_MODE_SHIFT)),
	IOMUX_PADS(PAD_ENET_RX_ER__ENET_RX_ER | MUX_PAD_CTRL(NO_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_REF_CLK__ENET_TX_CLK | MUX_PAD_CTRL(NO_PAD_CTRL)),
};

#if 0
static void setup_iomux_enet(void)
{
	imx_iomux_v3_setup_multiple_pads(enet_pads, ARRAY_SIZE(enet_pads));

	/* Reset AR8031 PHY */
/*	gpio_direction_output(IMX_GPIO_NR(1, 25) , 0);
	udelay(500);
	gpio_set_value(IMX_GPIO_NR(1, 25), 1);*/
	debug("IOMUX FEC MXC: %s:success\n", __func__);
}
#endif

static void setup_iomux_fec(void)
{
	struct iomuxc *iomux = (struct iomuxc *)IOMUXC_BASE_ADDR;
	u32 reg = 0;
	imx_iomux_v3_setup_multiple_pads(fec_pads, ARRAY_SIZE(fec_pads));

	/* Reset PHY */
/*	gpio_direction_output(IMX_GPIO_NR(1, 7) , 0);
	udelay(500);
	gpio_set_value(IMX_GPIO_NR(1, 7), 1);*/
	/* for Stella, not output clock, use input clock from PHY */
	reg = readl(&iomux->gpr[1]);
#define IMX6Q_GPR1_ENET_CLK_SEL (1 << 21)
	reg &= ~IMX6Q_GPR1_ENET_CLK_SEL;
	writel(reg, &iomux->gpr[1]);
#undef IMX6Q_GPR1_ENET_CLK_SEL
	debug("IOMUX FEC MXC: %s:success\n", __func__);
}

iomux_v3_cfg_t const usdhc3_pads[] = {
	IOMUX_PADS(PAD_SD3_CLK__SD3_CLK   | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD3_CMD__SD3_CMD   | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD3_DAT0__SD3_DATA0 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD3_DAT1__SD3_DATA1 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD3_DAT2__SD3_DATA2 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD3_DAT3__SD3_DATA3 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
};

iomux_v3_cfg_t const usdhc4_pads[] = {
	IOMUX_PADS(PAD_SD4_CLK__SD4_CLK   | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_CMD__SD4_CMD   | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_DAT0__SD4_DATA0 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_DAT1__SD4_DATA1 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_DAT2__SD4_DATA2 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_DAT3__SD4_DATA3 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_DAT4__SD4_DATA4 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_DAT5__SD4_DATA5 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_DAT6__SD4_DATA6 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
	IOMUX_PADS(PAD_SD4_DAT7__SD4_DATA7 | MUX_PAD_CTRL(USDHC_PAD_CTRL)),

	IOMUX_PADS(PAD_NANDF_ALE__SD4_RESET | MUX_PAD_CTRL(USDHC_PAD_CTRL)),
};

static void setup_iomux_uart(void)
{
	SETUP_IOMUX_PADS(uart1_pads);
}


static void setup_gpio(void)
{
	/*order: as Javelin1_Stella_Pinlist.xls*/
	gpio_request(IMX_GPIO_NR(5, 22), "");
	gpio_request(IMX_GPIO_NR(5, 23), "");
	gpio_request(IMX_GPIO_NR(5, 24), "");
	gpio_request(IMX_GPIO_NR(5, 25), "");
	gpio_direction_input(IMX_GPIO_NR(5, 22));
	gpio_direction_output(IMX_GPIO_NR(5, 23) , 0);
	gpio_direction_input(IMX_GPIO_NR(5, 24));
	gpio_direction_output(IMX_GPIO_NR(5, 25) , 0);

	gpio_request(IMX_GPIO_NR(5, 28), "");
	gpio_request(IMX_GPIO_NR(5, 29), "");
	gpio_direction_input(IMX_GPIO_NR(5, 28));
	gpio_direction_output(IMX_GPIO_NR(5, 29) , 0);

	gpio_request(IMX_GPIO_NR(6, 2), "");
	gpio_request(IMX_GPIO_NR(6, 3), "");
	gpio_request(IMX_GPIO_NR(6, 4), "");
	gpio_request(IMX_GPIO_NR(6, 5), "");
	gpio_direction_output(IMX_GPIO_NR(6, 2) , 0);
	gpio_direction_input(IMX_GPIO_NR(6, 3));
	gpio_direction_input(IMX_GPIO_NR(6, 4));
	gpio_direction_output(IMX_GPIO_NR(6, 5) , 0);

	gpio_request(IMX_GPIO_NR(5, 20), "");
	gpio_request(IMX_GPIO_NR(5, 19), "");
	gpio_request(IMX_GPIO_NR(5, 18), "");
	gpio_request(IMX_GPIO_NR(5, 21), "");
	gpio_direction_input(IMX_GPIO_NR(5, 20));
	gpio_direction_output(IMX_GPIO_NR(5, 19) , 0);
	gpio_direction_output(IMX_GPIO_NR(5, 18) , 0);
	gpio_direction_input(IMX_GPIO_NR(5, 21));

	gpio_request(IMX_GPIO_NR(4, 16), "");
	gpio_request(IMX_GPIO_NR(4, 17), "");
	gpio_request(IMX_GPIO_NR(4, 18), "");
	gpio_request(IMX_GPIO_NR(4, 19), "");
	gpio_request(IMX_GPIO_NR(4, 20), "");
	/*gpio_request(IMX_GPIO_NR(4, 24), "");*/ /*SPI3 CS1*/
	gpio_request(IMX_GPIO_NR(4, 25), "");
	gpio_request(IMX_GPIO_NR(4, 26), "");
	gpio_request(IMX_GPIO_NR(4, 27), "");
	gpio_request(IMX_GPIO_NR(4, 28), "");
	gpio_request(IMX_GPIO_NR(4, 29), "");
	gpio_request(IMX_GPIO_NR(4, 30), "");
	gpio_request(IMX_GPIO_NR(4, 31), "");
	gpio_direction_output(IMX_GPIO_NR(4, 16) , 0);
	gpio_direction_output(IMX_GPIO_NR(4, 17) , 0);
	gpio_direction_input(IMX_GPIO_NR(4, 18));
	gpio_direction_output(IMX_GPIO_NR(4, 19) , 0);
	gpio_direction_input(IMX_GPIO_NR(4, 20));
	/*gpio_direction_output(IMX_GPIO_NR(4, 24) , 1);*/ /*SPI3 CS1*/
	gpio_direction_output(IMX_GPIO_NR(4, 25) , 0);
	gpio_direction_output(IMX_GPIO_NR(4, 26) , 0);
	gpio_direction_output(IMX_GPIO_NR(4, 27) , 0);
	gpio_direction_input(IMX_GPIO_NR(4, 28));
	gpio_direction_output(IMX_GPIO_NR(4, 29) , 0);
	gpio_direction_output(IMX_GPIO_NR(4, 30) , 0);
	gpio_direction_output(IMX_GPIO_NR(4, 31) , 0);

	gpio_request(IMX_GPIO_NR(5, 5), "");
	gpio_request(IMX_GPIO_NR(5, 6), "");
	gpio_request(IMX_GPIO_NR(5, 7), "");
	gpio_request(IMX_GPIO_NR(5, 8), "");
	/*gpio_request(IMX_GPIO_NR(5, 9), "");*/ /*SPI2 CS2*/
	/*gpio_request(IMX_GPIO_NR(5, 12), "");*/ /*SPI2 CS1*/
	gpio_direction_input(IMX_GPIO_NR(5, 5));
	gpio_direction_output(IMX_GPIO_NR(5, 6) , 0);
	gpio_direction_output(IMX_GPIO_NR(5, 7) , 0);
	gpio_direction_input(IMX_GPIO_NR(5, 8));
	/*gpio_direction_output(IMX_GPIO_NR(5, 9) , 1);*/ /*SPI2 CS2*/
	/*gpio_direction_output(IMX_GPIO_NR(5, 12) , 1);*/ /*SPI2 CS1*/

	gpio_request(IMX_GPIO_NR(5, 14), "");
	gpio_request(IMX_GPIO_NR(5, 15), "");
	gpio_request(IMX_GPIO_NR(5, 16), "");
	gpio_request(IMX_GPIO_NR(5, 17), "");
	gpio_direction_output(IMX_GPIO_NR(5, 14) , 0);
	gpio_direction_output(IMX_GPIO_NR(5, 15) , 0);
	gpio_direction_output(IMX_GPIO_NR(5, 16) , 0);
	gpio_direction_output(IMX_GPIO_NR(5, 17) , 0);

	gpio_request(IMX_GPIO_NR(2, 22), "");
	gpio_request(IMX_GPIO_NR(2, 21), "");
	gpio_request(IMX_GPIO_NR(2, 20), "");
	gpio_request(IMX_GPIO_NR(2, 19), "");
	gpio_request(IMX_GPIO_NR(2, 18), "");
	gpio_request(IMX_GPIO_NR(2, 17), "");
	gpio_request(IMX_GPIO_NR(2, 16), "");
	gpio_direction_output(IMX_GPIO_NR(2, 22) , 0);
	gpio_direction_output(IMX_GPIO_NR(2, 21) , 0);
	gpio_direction_input(IMX_GPIO_NR(2, 20));
	gpio_direction_output(IMX_GPIO_NR(2, 19) , 0);
	gpio_direction_input(IMX_GPIO_NR(2, 18));
	gpio_direction_input(IMX_GPIO_NR(2, 17));
	gpio_direction_output(IMX_GPIO_NR(2, 16) , 0);

	gpio_request(IMX_GPIO_NR(6, 6), "");
	gpio_direction_input(IMX_GPIO_NR(6, 6));

	gpio_request(IMX_GPIO_NR(5, 4), "");
	gpio_request(IMX_GPIO_NR(5, 2), "");
	gpio_direction_output(IMX_GPIO_NR(5, 4) , 0);
	gpio_direction_output(IMX_GPIO_NR(5, 2) , 0);

	gpio_request(IMX_GPIO_NR(6, 31), "");
	gpio_direction_output(IMX_GPIO_NR(6, 31) , 0);

	gpio_request(IMX_GPIO_NR(2, 23), "");
	gpio_request(IMX_GPIO_NR(2, 24), "");
	gpio_direction_output(IMX_GPIO_NR(2, 23) , 0);
	gpio_direction_output(IMX_GPIO_NR(2, 24) , 0);

	gpio_request(IMX_GPIO_NR(3, 16), "");
	gpio_direction_output(IMX_GPIO_NR(3, 16) , 0);

	gpio_request(IMX_GPIO_NR(3, 19), "");
	gpio_direction_output(IMX_GPIO_NR(3, 19) , 0);

	/*gpio_request(IMX_GPIO_NR(3, 20), "");
	gpio_direction_output(IMX_GPIO_NR(3, 20) , 1);*/ /*SPI4 CS1*/

	gpio_request(IMX_GPIO_NR(3, 23), "");
	gpio_direction_output(IMX_GPIO_NR(3, 23) , 1);

	gpio_request(IMX_GPIO_NR(3, 26), "");
	gpio_request(IMX_GPIO_NR(3, 27), "");
	gpio_direction_output(IMX_GPIO_NR(3, 26) , 0);
	gpio_direction_output(IMX_GPIO_NR(3, 27) , 0);

	gpio_request(IMX_GPIO_NR(3, 29), "");
	gpio_request(IMX_GPIO_NR(3, 30), "");
	gpio_request(IMX_GPIO_NR(3, 31), "");
	gpio_direction_output(IMX_GPIO_NR(3, 29) , 0);
	gpio_direction_output(IMX_GPIO_NR(3, 30) , 0);
	gpio_direction_output(IMX_GPIO_NR(3, 31) , 0);

	gpio_request(IMX_GPIO_NR(3, 0), "");
	/*gpio_request(IMX_GPIO_NR(3, 1), "");*/ /*SPI4 CS3*/
	gpio_request(IMX_GPIO_NR(3, 2), "");
	gpio_request(IMX_GPIO_NR(3, 3), "");
	/*gpio_request(IMX_GPIO_NR(3, 4), "");*/ /*SPI2 CS3*/
	gpio_request(IMX_GPIO_NR(3, 5), "");
	/*gpio_request(IMX_GPIO_NR(3, 6), "");*/ /*SPI4 CS2*/
	gpio_request(IMX_GPIO_NR(3, 7), "");
	gpio_request(IMX_GPIO_NR(3, 8), "");
	gpio_request(IMX_GPIO_NR(3, 9), "");
	gpio_request(IMX_GPIO_NR(3, 10), "");
	/*gpio_request(IMX_GPIO_NR(3, 11), "");*/ /*SPI4 CS5*/
	/*gpio_request(IMX_GPIO_NR(3, 12), "");*/ /*SPI4 CS4*/
	gpio_request(IMX_GPIO_NR(3, 13), "");
	gpio_request(IMX_GPIO_NR(3, 14), "");
	gpio_request(IMX_GPIO_NR(3, 15), "");
	gpio_direction_output(IMX_GPIO_NR(3, 0) , 0);
	/*gpio_direction_output(IMX_GPIO_NR(3, 1) , 1);*/ /*SPI4 CS3*/
	gpio_direction_output(IMX_GPIO_NR(3, 2) , 0);
	gpio_direction_output(IMX_GPIO_NR(3, 3) , 0);
	/*gpio_direction_output(IMX_GPIO_NR(3, 4) , 1);*/ /*SPI2 CS3*/
	gpio_direction_output(IMX_GPIO_NR(3, 5) , 1);
	/*gpio_direction_output(IMX_GPIO_NR(3, 6) , 1);*/ /*SPI4 CS2*/
	gpio_direction_input(IMX_GPIO_NR(3, 7));
	gpio_direction_output(IMX_GPIO_NR(3, 8) , 0);
	gpio_direction_input(IMX_GPIO_NR(3, 9));
	gpio_direction_input(IMX_GPIO_NR(3, 10));
	/*gpio_direction_input(IMX_GPIO_NR(3, 11));*/ /*SPI4 CS5*/
	/*gpio_direction_input(IMX_GPIO_NR(3, 12));*/ /*SPI4 CS4*/
	gpio_direction_input(IMX_GPIO_NR(3, 13));
	gpio_direction_output(IMX_GPIO_NR(3, 14) , 1);
	gpio_direction_input(IMX_GPIO_NR(3, 15));

	gpio_request(IMX_GPIO_NR(2, 28), "");
	gpio_request(IMX_GPIO_NR(2, 29), "");
	gpio_request(IMX_GPIO_NR(2, 30), "");
	gpio_request(IMX_GPIO_NR(2, 31), "");
	gpio_request(IMX_GPIO_NR(2, 27), "");
	gpio_request(IMX_GPIO_NR(2, 25), "");
	gpio_request(IMX_GPIO_NR(2, 26), "");
	gpio_direction_input(IMX_GPIO_NR(2, 28));
	gpio_direction_input(IMX_GPIO_NR(2, 29));
	gpio_direction_input(IMX_GPIO_NR(2, 30));
	gpio_direction_input(IMX_GPIO_NR(2, 31));
	gpio_direction_input(IMX_GPIO_NR(2, 27));
	gpio_direction_output(IMX_GPIO_NR(2, 25) , 0);
	gpio_direction_input(IMX_GPIO_NR(2, 26));

	gpio_request(IMX_GPIO_NR(5, 0), "");
	gpio_direction_input(IMX_GPIO_NR(5, 0));

	gpio_request(IMX_GPIO_NR(1, 2), "");
	gpio_direction_output(IMX_GPIO_NR(1, 2) , 0);

	gpio_request(IMX_GPIO_NR(1, 4), "");
	gpio_request(IMX_GPIO_NR(1, 5), "");
	gpio_request(IMX_GPIO_NR(1, 6), "");
	gpio_request(IMX_GPIO_NR(1, 7), "");
	gpio_request(IMX_GPIO_NR(1, 8), "");
	gpio_direction_output(IMX_GPIO_NR(1, 4) , 0);
	gpio_direction_input(IMX_GPIO_NR(1, 5));
	gpio_direction_output(IMX_GPIO_NR(1, 6) , 0);
	gpio_direction_output(IMX_GPIO_NR(1, 7) , 0);
	gpio_direction_input(IMX_GPIO_NR(1, 8));

	gpio_request(IMX_GPIO_NR(7, 12), "");
	gpio_request(IMX_GPIO_NR(7, 13), "");
	gpio_direction_output(IMX_GPIO_NR(7, 12) , 0);
	gpio_direction_output(IMX_GPIO_NR(7, 13) , 0);

	/*gpio_request(IMX_GPIO_NR(4, 10), "");
	gpio_direction_output(IMX_GPIO_NR(4, 10) , 1);*/ /*SPI1 CS2*/

	gpio_request(IMX_GPIO_NR(4, 14), "");
	gpio_direction_output(IMX_GPIO_NR(4, 14) , 0);

	/*gpio_request(IMX_GPIO_NR(4, 9), "");*/ /*SPI1 CS1*/
	/*gpio_request(IMX_GPIO_NR(4, 11), "");*/ /*SPI1 CS3*/
	/*gpio_direction_output(IMX_GPIO_NR(4, 9) , 1);*/ /*SPI1 CS1*/
	/*gpio_direction_output(IMX_GPIO_NR(4, 11) , 1);*/ /*SPI1 CS3*/

	gpio_request(IMX_GPIO_NR(4, 15), "");
	gpio_direction_output(IMX_GPIO_NR(4, 15) , 0);

	gpio_request(IMX_GPIO_NR(6, 7), "");
	gpio_request(IMX_GPIO_NR(6, 11), "");
	gpio_request(IMX_GPIO_NR(6, 14), "");
	gpio_request(IMX_GPIO_NR(6, 15), "");
	gpio_request(IMX_GPIO_NR(6, 16), "");
	gpio_direction_output(IMX_GPIO_NR(6, 7) , 0);
	gpio_direction_input(IMX_GPIO_NR(6, 11));
	gpio_direction_output(IMX_GPIO_NR(6, 14) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 15) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 16) , 0);

	gpio_request(IMX_GPIO_NR(2, 0), "");
	gpio_request(IMX_GPIO_NR(2, 1), "");
	gpio_request(IMX_GPIO_NR(2, 2), "");
	gpio_request(IMX_GPIO_NR(2, 3), "");
	gpio_request(IMX_GPIO_NR(2, 4), "");
	gpio_request(IMX_GPIO_NR(2, 5), "");
	gpio_request(IMX_GPIO_NR(2, 6), "");
	gpio_request(IMX_GPIO_NR(2, 7), "");
	gpio_direction_output(IMX_GPIO_NR(2, 0) , 1);
	gpio_direction_output(IMX_GPIO_NR(2, 1) , 0);
	gpio_direction_output(IMX_GPIO_NR(2, 2) , 1);
	gpio_direction_input(IMX_GPIO_NR(2, 3));
	gpio_direction_input(IMX_GPIO_NR(2, 4));
	gpio_direction_output(IMX_GPIO_NR(2, 5) , 0);
	gpio_direction_output(IMX_GPIO_NR(2, 6) , 0);
	gpio_direction_output(IMX_GPIO_NR(2, 7) , 0);

	gpio_request(IMX_GPIO_NR(6, 10), "");
	gpio_request(IMX_GPIO_NR(6, 9), "");
	gpio_direction_output(IMX_GPIO_NR(6, 10) , 1);
	gpio_direction_input(IMX_GPIO_NR(6, 9));

	gpio_request(IMX_GPIO_NR(6, 25), "");
	gpio_request(IMX_GPIO_NR(6, 27), "");
	gpio_request(IMX_GPIO_NR(6, 28), "");
	gpio_request(IMX_GPIO_NR(6, 29), "");
	/*gpio_request(IMX_GPIO_NR(6, 24), "");*/ /*SPI3 CS2*/
	gpio_request(IMX_GPIO_NR(6, 30), "");
	gpio_request(IMX_GPIO_NR(6, 20), "");
	gpio_request(IMX_GPIO_NR(6, 21), "");
	gpio_request(IMX_GPIO_NR(6, 22), "");
	gpio_request(IMX_GPIO_NR(6, 23), "");
	gpio_request(IMX_GPIO_NR(6, 26), "");
	gpio_request(IMX_GPIO_NR(6, 19), "");
	gpio_direction_output(IMX_GPIO_NR(6, 25) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 27) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 28) , 0);
	gpio_direction_input(IMX_GPIO_NR(6, 29));
	/*gpio_direction_output(IMX_GPIO_NR(6, 24) , 1);*/ /*SPI3 CS2*/
	gpio_direction_output(IMX_GPIO_NR(6, 30) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 20) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 21) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 22) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 23) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 26) , 0);
	gpio_direction_output(IMX_GPIO_NR(6, 19) , 0);

	gpio_request(IMX_GPIO_NR(1, 20), "");
	gpio_direction_output(IMX_GPIO_NR(1, 20) , 0);

	gpio_request(IMX_GPIO_NR(1, 16), "");
	gpio_direction_output(IMX_GPIO_NR(1, 16) , 1);

	gpio_request(IMX_GPIO_NR(1, 19), "");
	/*gpio_request(IMX_GPIO_NR(1, 21), "");*/ /*SPI1 CS4*/
	gpio_request(IMX_GPIO_NR(1, 10), "");
	/*gpio_request(IMX_GPIO_NR(1, 11), "");*/ /*SPI2 CS4*/
	gpio_request(IMX_GPIO_NR(1, 15), "");
	gpio_direction_input(IMX_GPIO_NR(1, 19));
	/*gpio_direction_output(IMX_GPIO_NR(1, 21) , 1);*/ /*SPI1 CS4*/
	gpio_direction_input(IMX_GPIO_NR(1, 10));
	/*gpio_direction_output(IMX_GPIO_NR(1, 11) , 1);*/ /*SPI2 CS4*/
	gpio_direction_input(IMX_GPIO_NR(1, 15));

	gpio_request(IMX_GPIO_NR(7, 3), "");
	gpio_request(IMX_GPIO_NR(7, 2), "");
	gpio_request(IMX_GPIO_NR(7, 4), "");
	gpio_request(IMX_GPIO_NR(7, 5), "");
	gpio_request(IMX_GPIO_NR(7, 6), "");
	gpio_request(IMX_GPIO_NR(7, 7), "");
	gpio_direction_input(IMX_GPIO_NR(7, 3));
	gpio_direction_output(IMX_GPIO_NR(7, 2) , 1);
	gpio_direction_output(IMX_GPIO_NR(7, 4) , 0);
	gpio_direction_output(IMX_GPIO_NR(7, 5) , 0);
	gpio_direction_output(IMX_GPIO_NR(7, 6) , 0);
	gpio_direction_output(IMX_GPIO_NR(7, 7) , 1);

	gpio_request(IMX_GPIO_NR(7, 8), "");
	gpio_direction_input(IMX_GPIO_NR(7, 8));

	/*USB_H1_PWR*/
	gpio_request(IMX_GPIO_NR(1, 0), "");
	gpio_direction_output(IMX_GPIO_NR(1, 0) , 0);

}


void ldo_mode_set(int ldo_bypass){;}


#ifdef CONFIG_FSL_ESDHC
struct fsl_esdhc_cfg usdhc_cfg[CONFIG_SYS_FSL_USDHC_NUM] = {
	{USDHC4_BASE_ADDR},
	{USDHC3_BASE_ADDR},
};

int mmc_get_env_devno(void)
{
#if 0
	u32 soc_sbmr = readl(SRC_BASE_ADDR + 0x4);
	u32 dev_no;

	/* BOOT_CFG2[3] and BOOT_CFG2[4] */
	dev_no = (soc_sbmr & 0x00001800) >> 11;

	/* need ubstract 1 to map to the mmc device id
	 * see the comments in board_mmc_init function
	 */

	dev_no--;

	return dev_no;
#endif //#if 0
	return 0;
}


//#define USDHC1_CD_GPIO	IMX_GPIO_NR(4, 7)
//#define USDHC2_CD_GPIO	IMX_GPIO_NR(2, 2)
//#define USDHC3_CD_GPIO	IMX_GPIO_NR(2, 0)

int board_mmc_getcd(struct mmc *mmc)
{
	struct fsl_esdhc_cfg *cfg = (struct fsl_esdhc_cfg *)mmc->priv;
	int ret = 0;

	switch (cfg->esdhc_base) {
	case USDHC1_BASE_ADDR:
		ret = 0; /* not connected */
		break;
	case USDHC2_BASE_ADDR:
		ret = 0; /* not connected */
		break;
	case USDHC3_BASE_ADDR:
		ret = 0; /* SDIO but has no connect detection, never use in boot.*/
		break;
	case USDHC4_BASE_ADDR:
		ret = 1; /* eMMC/uSDHC4 is always present */
		break;
	}

	return ret;
}

int board_mmc_init(bd_t *bis)
{
	int i;

	/*
	 * According to the board_mmc_init() the following map is done:
	 * (U-boot device node)    (Physical Port)
	 * mmc0                    eMMC/USDHC4
	 * mmc1                    SDIO/USDHC3
	 */
	for (i = 0; i < CONFIG_SYS_FSL_USDHC_NUM; i++) {
		switch (i) {
		case 0:
			imx_iomux_v3_setup_multiple_pads(
				usdhc4_pads, ARRAY_SIZE(usdhc4_pads));
			usdhc_cfg[0].sdhc_clk = mxc_get_clock(MXC_ESDHC4_CLK);
			break;
		case 1:
			imx_iomux_v3_setup_multiple_pads(
				usdhc3_pads, ARRAY_SIZE(usdhc3_pads));
			usdhc_cfg[1].sdhc_clk = mxc_get_clock(MXC_ESDHC3_CLK);
			break;
		default:
			printf("Warning: you configured more USDHC controllers"
				"(%d) than supported by the board\n", i + 1);
			return 0;
	       }

	       if (fsl_esdhc_initialize(bis, &usdhc_cfg[i]))
			printf("Warning: failed to initialize mmc dev %d\n", i);
	}

	return 0;
}

#endif // CONFIG_FSL_ESDHC

int mx6_rgmii_rework(struct phy_device *phydev)
{
	unsigned short val;

	/* To enable AR8031 ouput a 125MHz clk from CLK_25M */
	phy_write(phydev, MDIO_DEVAD_NONE, 0xd, 0x7);
	phy_write(phydev, MDIO_DEVAD_NONE, 0xe, 0x8016);
	phy_write(phydev, MDIO_DEVAD_NONE, 0xd, 0x4007);

	val = phy_read(phydev, MDIO_DEVAD_NONE, 0xe);
	val &= 0xffe3;
	val |= 0x18;
	phy_write(phydev, MDIO_DEVAD_NONE, 0xe, val);

	/* introduce tx clock delay */
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1d, 0x5);
	val = phy_read(phydev, MDIO_DEVAD_NONE, 0x1e);
	val |= 0x0100;
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1e, val);

	return 0;
}

int board_phy_config(struct phy_device *phydev)
{
	//mx6_rgmii_rework(phydev);

	if (phydev->drv->config)
		phydev->drv->config(phydev);

	return 0;
}

#if defined(CONFIG_VIDEO_IPUV3)

static void enable_hdmi(struct display_info_t const *dev)
{
	struct hdmi_regs *hdmi	= (struct hdmi_regs *)HDMI_ARB_BASE_ADDR;
	u8 reg;
	printf("%s: setup HDMI monitor\n", __func__);
	reg = readb(&hdmi->phy_conf0);
	reg |= HDMI_PHY_CONF0_PDZ_MASK;
	writeb(reg, &hdmi->phy_conf0);

	udelay(3000);
	reg |= HDMI_PHY_CONF0_ENTMDS_MASK;
	writeb(reg, &hdmi->phy_conf0);
	udelay(3000);
	reg |= HDMI_PHY_CONF0_GEN2_TXPWRON_MASK;
	writeb(reg, &hdmi->phy_conf0);
	writeb(HDMI_MC_PHYRSTZ_ASSERT, &hdmi->mc_phyrstz);
}

/*static void disable_lvds(struct display_info_t const *dev)
{
	struct iomuxc *iomux = (struct iomuxc *)IOMUXC_BASE_ADDR;

	int reg = readl(&iomux->gpr[2]);

	reg &= ~(IOMUXC_GPR2_LVDS_CH0_MODE_MASK |
		 IOMUXC_GPR2_LVDS_CH1_MODE_MASK);

	writel(reg, &iomux->gpr[2]);
}

static void do_enable_hdmi(struct display_info_t const *dev)
{
	disable_lvds(dev);
	imx_enable_hdmi_phy();
}*/

struct display_info_t const displays[] = {{
	.bus	= -1,
	.addr	= 0,
	.pixfmt	= IPU_PIX_FMT_RGB24,
	.detect	= detect_hdmi,
	.enable	= enable_hdmi,
	.mode	= {
		.name           = "HDMI",
		.refresh        = 60,
		.xres           = 1920,
		.yres           = 1080,
		.pixclock       = 148500,
		.left_margin    = 148,
		.right_margin   = 88,
		.upper_margin   = 36,
		.lower_margin   = 4,
		.hsync_len      = 44,
		.vsync_len      = 5,
		.sync           = FB_SYNC_EXT,
		.vmode          = FB_VMODE_NONINTERLACED
} } };

static void setup_display(void)
{
	struct mxc_ccm_reg *mxc_ccm = (struct mxc_ccm_reg *)CCM_BASE_ADDR;
	/*struct iomuxc *iomux = (struct iomuxc *)IOMUXC_BASE_ADDR;*/
	int reg;

	enable_ipu_clock();
	/*if (is_mx6dqp()) {*/ /*from u-boot 2017.07, do force*/
#define MXC_CCM_CCGR6_PRG_CLK0_MASK		(3 << 24)
		setbits_le32(&mxc_ccm->CCGR6, MXC_CCM_CCGR6_PRG_CLK0_MASK);
		setbits_le32(&mxc_ccm->CCGR3, MXC_CCM_CCGR3_IPU2_IPU_MASK);
	/*}*/
	imx_setup_hdmi();

	/* Turn on LDB0, LDB1, IPU,IPU DI0 clocks */
	reg = __raw_readl(&mxc_ccm->CCGR3);
	reg |=  MXC_CCM_CCGR3_LDB_DI0_MASK | MXC_CCM_CCGR3_LDB_DI1_MASK;
	writel(reg, &mxc_ccm->CCGR3);
#if 0 /*maybe cause freeze before linux boot, so comment out*/
	/* set LDB0, LDB1 clk select to 011/011 */
	reg = readl(&mxc_ccm->cs2cdr);
	reg &= ~(MXC_CCM_CS2CDR_LDB_DI0_CLK_SEL_MASK
		 | MXC_CCM_CS2CDR_LDB_DI1_CLK_SEL_MASK);
	reg |= (3 << MXC_CCM_CS2CDR_LDB_DI0_CLK_SEL_OFFSET)
	      | (3 << MXC_CCM_CS2CDR_LDB_DI1_CLK_SEL_OFFSET);
	writel(reg, &mxc_ccm->cs2cdr);

	reg = readl(&mxc_ccm->cscmr2);
	reg |= MXC_CCM_CSCMR2_LDB_DI0_IPU_DIV | MXC_CCM_CSCMR2_LDB_DI1_IPU_DIV;
	writel(reg, &mxc_ccm->cscmr2);

	reg = readl(&mxc_ccm->chsccdr);
	reg &= ~(MXC_CCM_CHSCCDR_IPU1_DI0_PRE_CLK_SEL_MASK
		|MXC_CCM_CHSCCDR_IPU1_DI0_PODF_MASK
		|MXC_CCM_CHSCCDR_IPU1_DI0_CLK_SEL_MASK);
	reg |= (CHSCCDR_CLK_SEL_LDB_DI0
		<<MXC_CCM_CHSCCDR_IPU1_DI0_CLK_SEL_OFFSET)
	      |(CHSCCDR_PODF_DIVIDE_BY_3
		<<MXC_CCM_CHSCCDR_IPU1_DI0_PODF_OFFSET)
	      |(CHSCCDR_IPU_PRE_CLK_540M_PFD
		<<MXC_CCM_CHSCCDR_IPU1_DI0_PRE_CLK_SEL_OFFSET);
	writel(reg, &mxc_ccm->chsccdr);

	reg = IOMUXC_GPR2_BGREF_RRMODE_EXTERNAL_RES
	     | IOMUXC_GPR2_DI1_VS_POLARITY_ACTIVE_LOW
	     | IOMUXC_GPR2_DI0_VS_POLARITY_ACTIVE_LOW
	     | IOMUXC_GPR2_BIT_MAPPING_CH1_SPWG
	     | IOMUXC_GPR2_DATA_WIDTH_CH1_18BIT
	     | IOMUXC_GPR2_BIT_MAPPING_CH0_SPWG
	     | IOMUXC_GPR2_DATA_WIDTH_CH0_18BIT
	     | IOMUXC_GPR2_LVDS_CH0_MODE_DISABLED
	     | IOMUXC_GPR2_LVDS_CH1_MODE_ENABLED_DI0;
	writel(reg, &iomux->gpr[2]);

	reg = readl(&iomux->gpr[3]);
	reg = (reg & ~(IOMUXC_GPR3_LVDS1_MUX_CTL_MASK
			| IOMUXC_GPR3_HDMI_MUX_CTL_MASK))
	    | (IOMUXC_GPR3_MUX_SRC_IPU1_DI0
	       << IOMUXC_GPR3_LVDS1_MUX_CTL_OFFSET);
	writel(reg, &iomux->gpr[3]);
#endif /*#if 0*/
}
#endif /* CONFIG_VIDEO_IPUV3 */
/*
 * Do not overwrite the console
 * Use always serial for U-Boot console
 */
int overwrite_console(void)
{
	return 0;
}

static int setup_fec(void)
{
/*
	struct iomuxc_base_regs *const iomuxc_regs
		= (struct iomuxc_base_regs *) IOMUXC_BASE_ADDR;
	struct anatop_regs *anatop = (struct anatop_regs *)ANATOP_BASE_ADDR;
*/
	int ret;

	/* select anatop clock */
	/*int reg = readl(&iomuxc_regs->gpr[1]);
	reg |= (1 << 21);
	writel(reg, &iomuxc_regs->gpr[1]);*/

	//for LAN8720A
	/* Configure and Start the ENET PLL                                  */
	/*      diviser => 50 MHz                                            */
	/*writel (BM_ANADIG_PLL_ENET_DIV_SELECT, &anatop->pll_enet_clr);
	writel (0x1, &anatop->pll_enet_set);*/

	/*      clear bypass et powerdown                                    */
	/*writel (BM_ANADIG_PLL_ENET_BYPASS |  BM_ANADIG_PLL_ENET_POWERDOWN,
		&anatop->pll_enet_clr);*/
	/*      enable                                                       */
	/*writel (BM_ANADIG_PLL_ENET_ENABLE, &anatop->pll_enet_set);*/

	/* Reset PHY */
	/*gpio_direction_output(IMX_GPIO_NR(1, 7) , 0);
	mdelay(30);
	gpio_set_value(IMX_GPIO_NR(1, 7), 1);
	udelay(200);*/
	//for LAN8720A end

	ret = enable_fec_anatop_clock(0, ENET_50MHZ);
	// ret = enable_fec_clock();
	//ret = enable_fec_anatop_clock();
	if (ret)
		return ret;

	return 0;
}

int board_eth_init(bd_t *bis)
{
	int ret;

	setup_iomux_fec(); //for RMII
	//setup_iomux_enet(); //for RGMII


	ret = cpu_eth_init(bis);
	if (ret)
		printf("FEC MXC: %s:failed\n", __func__);
	debug("FEC MXC: %s:success\n", __func__);

	return 0;
}

int board_early_init_f(void)
{
	setup_iomux_uart();

#if defined(CONFIG_VIDEO_IPUV3)
	setup_display();
#endif

	return 0;
}

int board_init(void)
{
	/* address of boot parameters */
	gd->bd->bi_boot_params = PHYS_SDRAM + 0x100;

	setup_gpio();

#ifdef	CONFIG_FEC_MXC
	setup_fec();
#endif
	debug("%s:success\n", __func__);

	return 0;
}

#ifdef CONFIG_CMD_BMODE
static const struct boot_mode board_boot_modes[] = {
	/* 4 bit bus width */
	{"sd2",	 MAKE_CFGVAL(0x40, 0x28, 0x00, 0x00)},
	{"sd3",	 MAKE_CFGVAL(0x40, 0x30, 0x00, 0x00)},
	/* 8 bit bus width */
	{"emmc", MAKE_CFGVAL(0x40, 0x38, 0x00, 0x00)},
	{NULL,	 0},
};
#endif

int board_late_init(void)
{
#ifdef CONFIG_ENV_IS_IN_MMC
	board_late_mmc_env_init();
#endif

	return 0;
}

int checkboard(void)
{
	puts("Board: MX6DL-SonyStella Raptor 202011101800_DDRx2\n");

	return 0;
}

#ifdef CONFIG_USB_EHCI_MX6
iomux_v3_cfg_t const usb_otg_pads[] = {
	IOMUX_PADS(PAD_EIM_D22__USB_OTG_PWR | MUX_PAD_CTRL(NO_PAD_CTRL)),
	IOMUX_PADS(PAD_ENET_RX_ER__USB_OTG_ID | MUX_PAD_CTRL(NO_PAD_CTRL)),
};

iomux_v3_cfg_t const usb_hc1_pads[] = {
	IOMUX_PADS(PAD_ENET_TXD1__GPIO1_IO29 | MUX_PAD_CTRL(NO_PAD_CTRL)),
};

int board_ehci_hcd_init(int port)
{
	switch (port) {
	case 0:
		imx_iomux_v3_setup_multiple_pads(usb_otg_pads,
			ARRAY_SIZE(usb_otg_pads));

		/*set daisy chain for otg_pin_id on 6q. for 6dl, this bit is reserved*/
		imx_iomux_set_gpr_register(1, 13, 1, 0);
		break;
	case 1:
		imx_iomux_v3_setup_multiple_pads(usb_hc1_pads,
			ARRAY_SIZE(usb_hc1_pads));
		break;
	default:
		printf("MXC USB port %d not yet supported\n", port);
		return 1;
	}
	return 0;
}

int board_ehci_power(int port, int on)
{
	switch (port) {
	case 0:
		break;
	case 1:
		if (on)
			gpio_direction_output(IMX_GPIO_NR(1, 29), 1);
		else
			gpio_direction_output(IMX_GPIO_NR(1, 29), 0);
		break;
	default:
		printf("MXC USB port %d not yet supported\n", port);
		return 1;
	}
	return 0;
}

#endif

#if defined(CONFIG_ARCH_MISC_INIT)
int arch_misc_init(void)
{
    return 0;
}
#endif

